<?php

namespace Lavalite\Demo\Http\Controllers;

use App\Http\Controllers\UserController as UserController;
use Form;
use Lavalite\Demo\Http\Requests\DemoUserRequest;
use Lavalite\Demo\Interfaces\DemoRepositoryInterface;
use Lavalite\Demo\Models\Demo;

class DemoUserController extends UserController
{
    /**
     * Initialize demo controller.
     *
     * @param type DemoRepositoryInterface $demo
     *
     * @return type
     */
    public function __construct(DemoRepositoryInterface $demo)
    {
        $this->repository = $demo;
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return Response
     */
    public function index(DemoUserRequest $request)
    {
        $this->repository->pushCriteria(new \Lavalite\Demo\Repositories\Criteria\DemoUserCriteria());
        $demos = $this->repository->scopeQuery(function($query){
            return $query->orderBy('id','DESC');
        })->paginate();

        $this->theme->prependTitle(trans('demo::demo.names').' :: ');

        return $this->theme->of('demo::user.demo.index', compact('demos'))->render();
    }

    /**
     * Display the specified resource.
     *
     * @param Request $request
     * @param Demo $demo
     *
     * @return Response
     */
    public function show(DemoUserRequest $request, Demo $demo)
    {
        Form::populate($demo);

        return $this->theme->of('demo::user.demo.show', compact('demo'))->render();
    }

    /**
     * Show the form for creating a new resource.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(DemoUserRequest $request)
    {

        $demo = $this->repository->newInstance([]);
        Form::populate($demo);

        return $this->theme->of('demo::user.demo.create', compact('demo'))->render();
    }

    /**
     * Display the specified resource.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DemoUserRequest $request)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $demo = $this->repository->create($attributes);

            $this->responseCode = 201;
            $this->responseMessage = trans('messages.success.created', ['Module' => trans('demo::demo.name')]);
            $this->responseRedirect = trans_url('/user/demo/demo');
            return $this -> respond($request);
        } catch (Exception $e) {
            redirect()->back()->withInput()->with('message', $e->getMessage())->with('code', 400);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param Request $request
     * @param Demo $demo
     *
     * @return Response
     */
    public function edit(DemoUserRequest $request, Demo $demo)
    {

        Form::populate($demo);

        return $this->theme->of('demo::user.demo.edit', compact('demo'))->render();
    }

    /**
     * Update the specified resource.
     *
     * @param Request $request
     * @param Demo $demo
     *
     * @return Response
     */
    public function update(DemoUserRequest $request, Demo $demo)
    {
        try {
            $this->repository->update($request->all(), $demo->getRouteKey());
            $this->responseCode = 204;
            $this->responseMessage = trans('messages.success.updated', ['Module' => trans('demo::demo.name')]);
            $this->responseRedirect = trans_url('/user/demo/demo');
            return $this -> respond($request);
        } catch (Exception $e) {
            redirect()->back()->withInput()->with('message', $e->getMessage())->with('code', 400);
        }
    }

    /**
     * Remove the specified resource.
     *
     * @param int $id
     *
     * @return Response
     */
    public function destroy(DemoUserRequest $request, Demo $demo)
    {
        try {
            $this->repository->delete($demo->getRouteKey());
            $this->responseCode = 204;
            $this->responseMessage = trans('messages.success.deleted', ['Module' => trans('demo::demo.name')]);
            $this->responseRedirect = trans_url('/user/demo/demo');
            return $this -> respond($request);
        } catch (Exception $e) {

            redirect()->back()->withInput()->with('message', $e->getMessage())->with('code', 400);

        }
    }
}
